/*____________________________________________________________________________
	
	Voyeur - a Windows Network Crawler

	by Jawed Karim, 1999. http://www.jawed.com/

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
____________________________________________________________________________*/

#include <afxwin.h>
#include "stdafx.h"
#include <windows.h>
#include <winnetwk.h>
#include <stdio.h>
#include <iostream.h>
#include "network.h"

#include "Voyeur.h"
#include "VoyeurView.h"
#include "MainFrm.h"

#define MAXENTRIES	5

CVoyeurView *cvpGlobalViewPtr = NULL;

unsigned int StartMP3Search (void *p)
{
	// pass handle instead of CWnd ptr
	CWnd *pWnd = CWnd::FromHandle ((HWND)p);
	cvpGlobalViewPtr = (CVoyeurView *)pWnd;
	if (cvpGlobalViewPtr == NULL)
		return 0;

	HANDLE hEnumHandle;

	if ((hEnumHandle = GetNetworkHandle (NULL)) == NULL)
	{
		AfxMessageBox ("Done searching Network.");
		return 0;
	}

	RecurseNetworkLevels (hEnumHandle);
	WNetCloseEnum (hEnumHandle);

	AfxMessageBox ("Done searching Network.");
	return 1;
}

void RecurseNetworkLevels (HANDLE hStart)
{
	if (hStart == NULL)
		return;

	NETRESOURCE nrNetInfo[MAXENTRIES];

	int numEntries;
	
	while ((numEntries = GetNetworkInfo (hStart, nrNetInfo, MAXENTRIES - 1)) > 0)
	{
		if (numEntries <= 0)
			return;

		if (numEntries > MAXENTRIES)
			numEntries = MAXENTRIES;

		for (int i = 0; i < numEntries; i++)
		{
			ProcessNetresourceEntry (nrNetInfo[i]);

			HANDLE hTmp = GetNetworkHandle (&nrNetInfo[i]);
			RecurseNetworkLevels (hTmp);
			WNetCloseEnum (hTmp);
		}
	}
}

void ProcessNetresourceEntry (NETRESOURCE current)
{
	char *szResourceName = current.lpRemoteName;

	if (szResourceName == NULL)
		return;

	DWORD dwType = current.dwType;

	if (dwType == RESOURCETYPE_PRINT || dwType == RESOURCETYPE_DISK || dwType == RESOURCETYPE_ANY)
		ProcessDirectoryName (szResourceName);
}

void ProcessDirectoryName (char *dir)
{
	// ASSUME that 'dir' is a directory (no filenames coming in here)
	// incoming dirName doesn't have terminating "\"
	// we must append "\*.*" to specify which filenames to list (all because we want dirs too)

	char szDirName[MAX_PATH];

	strcpy (szDirName, dir);
	strcat (szDirName, "\\*.*");

	WIN32_FIND_DATA fd;

	HANDLE hFind = FindFirstFile (szDirName, &fd);

	if (hFind == INVALID_HANDLE_VALUE)
		return;

	while (FindNextFile (hFind, &fd))
	{
		char *szLocalName = fd.cFileName;

		if (fd.dwFileAttributes == FILE_ATTRIBUTE_DIRECTORY)
		{
			if ((strcmp (szLocalName, ".") != 0) && (strcmp (szLocalName, "..") != 0))
			{
				char szFullName[MAX_PATH];
				strcpy (szFullName, dir);			// "\\COMPUTER"
				strcat (szFullName, "\\");			// "\\COMPUTER\"
				strcat (szFullName, szLocalName);	// "\\COMPUTER\file"

				ProcessDirectoryName (szFullName);
			}
		}

		// this is a real filename
		else
		{
			char szFullName[MAX_PATH];
 
			// this is the full name (with path) of the file
			sprintf (szFullName, "%s\\%s", dir, szLocalName);

			// check for ".mp3" anywhere in the string
			if (strstr (szLocalName, ".mp3") != NULL)
			{
				cvpGlobalViewPtr->InsertFileItem (szLocalName, szFullName);
				::Sleep (120);
			}
		}
	}

	FindClose (hFind);
}

HANDLE GetNetworkHandle (NETRESOURCE *nsInit)
{
	HANDLE hEnumHandle = NULL;
	DWORD dwSuccess;

	dwSuccess = WNetOpenEnum (RESOURCE_GLOBALNET,
							  RESOURCETYPE_ANY,
							  0,
							  nsInit,
							  &hEnumHandle);  // return this handle

	if (dwSuccess == NO_ERROR)
		return hEnumHandle;

	else
	{
		switch (dwSuccess)
		{
			case ERROR_NO_NETWORK:
				AfxMessageBox ("ERROR: Unable to detect network. You are either not connected to a network\n"
						   "or your network is misconfigured."
						   "\n\nIf you ARE on a network, the following have proven to be solutions to\n"
						   "this error:\n\n"
						   "- Make sure you have removed any unecessary network protocols from your\n"
						   "  network settings. For example, if you are on a TCPIP network,\n"
						   "  remove references to IPX and other protocols.\n\n"
						   "- Disable DHCP and instead enter a valid WINS server for your network\n"
						   "  in your network settings.");
				break;

			case ERROR_EXTENDED_ERROR:
				break;

			default:
				break;
		}
		return NULL;
	}
}

int GetNetworkInfo (HANDLE hHandle, NETRESOURCE *nrNetInfo, DWORD cNumDevices)
{
	DWORD cSize = cNumDevices * sizeof (NETRESOURCE);
	DWORD dwSuccess;

	if (hHandle == NULL)
		return 0;

	dwSuccess = WNetEnumResource (hHandle,
								  &cNumDevices,
								  nrNetInfo,
								  &cSize);

	if (dwSuccess != ERROR_NO_MORE_ITEMS && dwSuccess != NO_ERROR)
		return -1;
	else if (dwSuccess == ERROR_NO_MORE_ITEMS)
		return 0;
	else
		return cNumDevices;
}
