/*____________________________________________________________________________
	
	Voyeur - a Windows Network Crawler

	by Jawed Karim, 1999. http://www.jawed.com/

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
____________________________________________________________________________*/

// VoyeurView.cpp : implementation of the CVoyeurView class
//

#include "stdafx.h"
#include "Voyeur.h"

#include "VoyeurDoc.h"
#include "VoyeurView.h"
#include "MainFrm.h"

#include "network.h"
#include "searchdialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CVoyeurView

IMPLEMENT_DYNCREATE (CVoyeurView, CListView)

BEGIN_MESSAGE_MAP (CVoyeurView, CListView)
	ON_WM_CREATE ()
	ON_WM_DESTROY ()

	ON_COMMAND (IDM_START_SEARCH, OnStartSearch)
	ON_COMMAND (IDM_SEARCH_LIST, OnFindInList)

	ON_COMMAND (ID_EXECUTE_FILE, OnExecuteFile)
	ON_COMMAND (ID_OPEN_FOLDER, OnOpenFolder)

	ON_COMMAND (IDM_HELP, OnClickHelp)

	// receive a double click
	ON_NOTIFY_REFLECT (NM_DBLCLK, OnItemDoubleClicked)

END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CVoyeurView construction/destruction

CVoyeurView::CVoyeurView()
{
	m_pSearchThread = NULL;
	m_hThreadHandle = NULL;
}

CVoyeurView::~CVoyeurView()
{
}

void CVoyeurView::OnClickHelp ()
{
	AfxMessageBox ("Voyeur -- a Windows Network Crawler\n\n"
				   "by Jawed Karim, 1999.\n"
				   "http://www.jawed.com/",
					MB_ICONINFORMATION);
}

void CVoyeurView::OnItemDoubleClicked (LPNMHDR *pNmhdr, LRESULT *lResult)
{
	int iSelected = GetSelectedItem ();

	if (iSelected < 0)
		return;

	char szPath[1024];
	GetListCtrl ().GetItemText (iSelected, 1, szPath, 1024);

	// launch app!
	LaunchFile (m_hWnd, szPath);
}

BOOL CVoyeurView::LaunchFile (HWND hParent, char *szPath)
{
	// launch the file with its associated application
	HINSTANCE hInst = ::ShellExecute (hParent, "open", szPath, NULL, "C:\\", SW_SHOWDEFAULT);

	if ((int)hInst > 32)
		return TRUE;

	switch ((int)hInst)
	{
		case SE_ERR_PNF:
		case SE_ERR_FNF:

			AfxMessageBox ("ERROR: The specified path was not found.");
			return FALSE;

		case SE_ERR_ACCESSDENIED:
		case SE_ERR_SHARE:

			AfxMessageBox ("ERROR: The operating system denied access to the specified file.");
			return FALSE;

		case SE_ERR_NOASSOC:
		case SE_ERR_ASSOCINCOMPLETE:

			AfxMessageBox ("ERROR: There is no application associated with the given filename extension.");
			return FALSE;

		default:
			return FALSE;
	}
}

void CVoyeurView::DeleteAllListItems ()
{
	GetListCtrl ().DeleteAllItems ();
}

int CVoyeurView::GetNumListItems ()
{
	return GetListCtrl ().GetItemCount ();
}

int CVoyeurView::GetSelectedItem ()
{
	// which item is selected?
	return (GetListCtrl ().GetNextItem (-1, LVIS_SELECTED));
}

BOOL CVoyeurView::GetFileItemText (int index, char *name, char *path)
{
	if (index >= GetNumListItems())
		return FALSE;
	if ((name == NULL) || (path == NULL))
		return FALSE;

	GetListCtrl ().GetItemText (index, 0, name, 255);
	GetListCtrl ().GetItemText (index, 1, path, 255);

	return TRUE;
}

BOOL CVoyeurView::InsertFileItem (char *name, char *path)
{
	if (name == NULL || path == NULL)
		return FALSE;

	int iNumItems = GetNumListItems ();

	// insert name
	LV_ITEM item;
	item.mask = LVIF_TEXT | LVIF_IMAGE | LVIF_PARAM;
	item.iItem = iNumItems;
	item.iSubItem = 0;
	item.pszText = name;
	item.cchTextMax = strlen (name);
	item.lParam = (LPARAM) iNumItems;

	item.iImage = 0;

	int iSuccess = GetListCtrl ().InsertItem (&item);

	if (iSuccess == -1)
		return FALSE;

	GetListCtrl ().SetItemText (iNumItems, 1, path);

	return TRUE;
}

BOOL CVoyeurView::ThreadRunning ()
{
	DWORD dwExitCode;
	::GetExitCodeThread (m_hThreadHandle, &dwExitCode);

	if (dwExitCode == STILL_ACTIVE)
		return TRUE;
	else
		return FALSE;
}

void CVoyeurView::OnOpenFolder ()
{
	int iNumItems = GetNumListItems ();
	int iSelected = -1;

	iSelected = GetSelectedItem ();

	if (iSelected < 0)
		return;

	char szPath[1024];
	char szFile[1024];

	GetListCtrl ().GetItemText (iSelected, 1, szPath, 1024);
	GetListCtrl ().GetItemText (iSelected, 0, szFile, 1024);

	strlwr (szPath);
	strlwr (szFile);

	char *pEnd = strstr (szPath, szFile);

	if (pEnd == NULL)
		return;

	*pEnd = '\0';

	LaunchFile (m_hWnd, szPath);
}

void CVoyeurView::OnExecuteFile ()
{
	int iNumItems = GetNumListItems ();
	int iSelected = -1;

	iSelected = GetSelectedItem ();

	if (iSelected < 0)
		return;

	char szPath[1024];
	GetListCtrl ().GetItemText (iSelected, 1, szPath, 1024);

	LaunchFile (m_hWnd, szPath);
}

void CVoyeurView::SelectItem  (int iIndex)
{
	GetListCtrl ().SetItemState (iIndex, LVIS_SELECTED | LVIS_FOCUSED, LVIS_SELECTED | LVIS_FOCUSED);

	// make sure it's visible
	GetListCtrl ().EnsureVisible (iIndex, FALSE);
}

int CVoyeurView::FindItem (char *szString, int iStartIndex)
{
	int iNumItems = GetNumListItems ();

	for (int i = iStartIndex; i < iNumItems; i++)
	{
		static char szFileName[1024];
		GetListCtrl ().GetItemText (i, 0, szFileName, 1024);

		if (strstr (_strlwr (szFileName), _strlwr (szString)) != NULL)
			return i;
	}

	// nothing found
	return -1;
}

void CVoyeurView::OnFindInList ()
{
	CSearchDialog dlg (this);

	int iSelected = GetSelectedItem () + 1;

	if (dlg.DoModal () == IDOK)
	{
		int iFoundIndex;

		iFoundIndex = FindItem (dlg.m_findtext, iSelected);

		if (iFoundIndex != -1)
		{
			SelectItem (iFoundIndex);
			return;
		}

		iFoundIndex = FindItem (dlg.m_findtext, 0);

		if (iFoundIndex != -1)
		{
			SelectItem (iFoundIndex);
			return;
		}

		AfxMessageBox ("String not found.");
	}
}

void CVoyeurView::OnStartSearch ()
{
	BOOL bAlreadyRunning = ThreadRunning ();

	if (bAlreadyRunning)
	{
		AfxMessageBox ("There's another search already running!");
		return;
	}

	int iNumItems = GetNumListItems();

	if (iNumItems > 0)
	{
		int iResponse;
		
		iResponse = MessageBox ("Running a new search will delete all files currently listed,\n"
								"and create a new current list. Are you sure you want to proceed?",
								"Warning", MB_YESNO | MB_ICONQUESTION);

		if (iResponse == IDNO)
			return;
		else DeleteAllListItems ();

	}

	m_pSearchThread = AfxBeginThread (StartMP3Search, m_hWnd);
	ASSERT (m_pSearchThread);

	m_hThreadHandle = m_pSearchThread->m_hThread;
	ASSERT (m_hThreadHandle);
}

////////////////////////////////////////////////////////////////////////////////////

void CVoyeurView::OnInitialUpdate ()
{
	CListView::OnInitialUpdate ();
}

void CVoyeurView::OnDestroy ()
{
	CListView::OnDestroy ();
}

int CVoyeurView::OnCreate (LPCREATESTRUCT lpcs)
{
	if (CListView::OnCreate (lpcs) == -1)
		return -1;

	// set up image list
	m_ImageListBig.Create (IDB_LARGE_BM, 32, 1, RGB (255, 255, 255));
	m_ImageListSmall.Create (IDB_SMALL_BM, 16, 1, RGB (255, 255, 255));

	GetListCtrl ().SetImageList (&m_ImageListBig, LVSIL_NORMAL);
	GetListCtrl ().SetImageList (&m_ImageListSmall, LVSIL_SMALL);

	// set up list control
	GetListCtrl ().InsertColumn (0, "Filename", LVCFMT_LEFT, 300);
	GetListCtrl ().InsertColumn (1, "Path", LVCFMT_LEFT, 300);

	///////////////////////////////////////////////
	return 0;
}

BOOL CVoyeurView::PreCreateWindow (CREATESTRUCT &cs)
{
	if (!CListView::PreCreateWindow (cs))
		return FALSE;

	// modify style to change to CListView. allow only single selection.
	cs.style |= LVS_REPORT | LVS_SINGLESEL | LVS_SHOWSELALWAYS;

	return TRUE;
}

#ifdef _DEBUG

void CVoyeurView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CVoyeurDoc* CVoyeurView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CVoyeurDoc)));
	return (CVoyeurDoc*)m_pDocument;
}
#endif //_DEBUG
