//
// Main.cpp
//
// Example of how to use CLdapFilter

#include "filter.h"
#include <iostream>
#include <cstdio>

using namespace std;

void TestFilter(wchar_t * sz)
{
    static wstring wsCaption = L"Original Filter : ";
    CLdapFilter MyFilter;
    size_t sPos = PARSER_SUCCESS;
    wstring wsLastError;

#ifndef LDAPFILTER_SUPPORT_EXCEPTION
    MyFilter = sz;
    sPos = MyFilter.GetLastErrorPos();
    if(sPos!=PARSER_SUCCESS)
	wsLastError = MyFilter.GetLastErrorString();
#else
    try {
	MyFilter = sz;
    } catch(CFilterException &fe) {
	sPos = fe.GetPos();
	wsLastError = fe.GetError();
    } catch(...) {
	wprintf(L"Unhandled exception.");
    }
#endif
    
    COut(wsCaption);
    if(sz)
	COut(sz);
    cout << endl;


    if(sPos == PARSER_SUCCESS) {
	wprintf(L"Result Filter   : %s\n", MyFilter.GetCString());
    } else {
	wstring wsErrorPos(wsCaption.size() + sPos, L' ');
	wsErrorPos += L'^';
	wprintf(L"%s - %s\n", wsErrorPos.c_str(), wsLastError.c_str());
    }

    wprintf(L"\n");

}


void SyntaxCheck()
{

    CLdapFilter MyFilter;

    // Good filters
    wprintf(L"*** Good Filters\n");
    TestFilter(L"obj=XX"); // Ok
    TestFilter(L"(obj=XX)"); // Ok
    TestFilter(L"((obj=123))"); // Ok
    TestFilter(L"(!(obj=123))"); // Ok
    TestFilter(L"(&(obj=XX)(user=YY))"); // Ok
    TestFilter(L"(!(&(obj=XX)(user=YY)))");
    TestFilter(L"(|(!(obj=XX))(&(user=YY)(obj=ZZ)))");
    TestFilter(L" ( & ( obj = 3 ) ( tw = 4 ) ) ");
    TestFilter(L"(&(|(ab<=4)(ab>=0))(zt=3)(zx~=7))");

    // Weird, but good
    wprintf(L"*** Good, but weird\n");
    TestFilter(NULL);
    TestFilter(L"");
    TestFilter(L"(obj==)");
    TestFilter(L"(a=\\()");
    TestFilter(L"(\\(=\\()");
    TestFilter(L"(a=\\27\\f4)");

    TestFilter(L"(&(&(a=b)(b=c))(c=d))");
    TestFilter(L"(&(a=b)(&(b=c)(c=d)))");
    TestFilter(L"(&(&(a=b)(b=c))(&(c=d)(d=e)))");
    TestFilter(L"(|(|(a=b)(b=c))(c=d))");
    TestFilter(L"(|(a=b)(|(b=c)(c=d)))");
    TestFilter(L"(|(|(a=b)(b=c))(|(c=d)(d=e)))");
    TestFilter(L"(!(!(a=b)))");
    TestFilter(L"(!(!(!(a=b))))");
    TestFilter(L"(!(!(!(!(a=b)))))");
    TestFilter(L"(&obj=3)");
    TestFilter(L"&obj = 4");
    TestFilter(L"obj|=4");
    TestFilter(L"(a=b<=3)");
    TestFilter(L"(a<=b=c)");


    wprintf(L"\n\n*** BAD filters\n");
    
    // Bad filters
    TestFilter(L"(a=   )"); // ?
    TestFilter(L"(!(!)  ");
    TestFilter(L"abc");
    TestFilter(L"abc\\=12");
    TestFilter(L"ab(");
    TestFilter(L"ab(ab=cd)");
    TestFilter(L"(obj=XX");
    TestFilter(L"obj=XX)");
    TestFilter(L"((obj=XX)");
    TestFilter(L"(obj=Y)a");
    TestFilter(L"(!(obj=XX)(obj=ZZ))");
    TestFilter(L"(&(obj=XX))"); // This work on NTDS
    TestFilter(L"(obj=XX\\)");
    TestFilter(L"()");
    TestFilter(L"(a=b)\\");
    TestFilter(L"(a=\\?)");
    TestFilter(L"(a=\\g3)");
    TestFilter(L"(<=b)");
    TestFilter(L"(a<b)");
    TestFilter(L"(=b)");
    TestFilter(L"(a(a=b))");
    TestFilter(L"(a=)");
    TestFilter(L"(a=b))");
    TestFilter(L"(a=b)c");
    TestFilter(L"((a=b)");
    TestFilter(L"(a)");
    TestFilter(L"(   =b)");
    TestFilter(L"(a b=c)");
    TestFilter(L"(&\\(a=b))");
    TestFilter(L"(a=b\\)");
    TestFilter(L"a(a=b)");
    TestFilter(L"(!(a=b)(c=d))");
    TestFilter(L"(!(!\\() ");
    TestFilter(L"=");
    TestFilter(L"=ab");

}


void main()
{

    SyntaxCheck();
 
}