/******************************************************************************

  Yet another registry access class

                      Copyright 1999 Gavin Smyth
                 http://www.beesknees.freeserve.co.uk

******************************************************************************/

#include "common.h"
#include "registry.h"

RegistryKey::RegistryKey( HKEY base, const TCHAR* keyName )
{
  if( RegCreateKeyEx( base, keyName, 0, _T(""), 0,
                      KEY_QUERY_VALUE | KEY_SET_VALUE,
                      NULL, &key, NULL ) != ERROR_SUCCESS )
    key = NULL;
}

RegistryKey::~RegistryKey()
{
  if( key != NULL )
  {
    RegCloseKey( key );
    key = NULL;
  }
}

void RegistryKey::Read( const TCHAR* value, TCHAR* data, unsigned long maxSize, const TCHAR* deflt ) const
{
  if( !ReadData( value, data, maxSize, REG_SZ ) )
    _tcsnccpy( data, deflt, maxSize );
  // Ensure null termination
  data[ maxSize - 1 ] = 0;
}

void RegistryKey::Write( const TCHAR* value, const TCHAR* data ) const
{
  WriteData( value, data, _tcslen( data ) + 1, REG_SZ ); // Take \0 into account
}

unsigned long RegistryKey::Read( const TCHAR* value, unsigned long deflt ) const
{
  unsigned long data;
  unsigned long size = sizeof( data );
  if( !ReadData( value, &data, size, REG_DWORD ) )
    return deflt;
  else
    return data;
}

void RegistryKey::Write( const TCHAR* value, unsigned long data ) const
{
  WriteData( value, &data, sizeof( data ), REG_DWORD );
}

void RegistryKey::Read( const TCHAR* value, void* data, unsigned long desiredSize, const void* deflt ) const
{
  unsigned long size = desiredSize;
  if( !ReadData( value, data, size, REG_BINARY ) && size == desiredSize )
    CopyMemory( data, deflt, desiredSize );
}

void RegistryKey::Write( const TCHAR* value, const void* data, unsigned long size ) const
{
  WriteData( value, data, size, REG_BINARY );
}

bool RegistryKey::ReadData( const TCHAR* value, void* data, unsigned long& size, unsigned long desiredType ) const
{
  if( key == NULL )
    return false;

  unsigned long type;
  return RegQueryValueEx( key, value, 0, &type,
                          static_cast< BYTE* >( data ),
                          &size ) == ERROR_SUCCESS &&
         type == desiredType;
}

bool RegistryKey::WriteData( const TCHAR* value, const void* data, unsigned long size, unsigned long type ) const
{
  if( key == NULL )
    return false;

  return RegSetValueEx( key, value, 0, type,
                        static_cast< CONST BYTE* >( data ),
                        size ) == ERROR_SUCCESS;
}
