/******************************************************************************

  Yet another registry access class

  Fairly restricted - I didn't bother with enumerating or anything beyond
  simple data types.

                      Copyright 1999 Gavin Smyth
                 http://www.beesknees.freeserve.co.uk

******************************************************************************/

#ifndef _h_registry_h_
#define _h_registry_h_

class RegistryKey
{
public:
  // Open (or create, if necessary) the key below the specified handle
  RegistryKey( HKEY base, const TCHAR* keyName );

  // Close the key
  ~RegistryKey();

  // The read routines below take a default parameter, used if the
  // value isn't found in the registry (or there was an error in
  // opening the key - for my app that's more convenient than bothering
  // to handle that error condition).

  // Read and write character strings
  void Read( const TCHAR* value, TCHAR* data, unsigned long maxSize, const TCHAR* deflt  ) const;
  void Write( const TCHAR* value, const TCHAR* data ) const;

  // Read and write long integers
  unsigned long Read( const TCHAR* value, unsigned long deflt ) const;
  void Write( const TCHAR* value, unsigned long data ) const;

  // Read and write binary data
  void Read( const TCHAR* value, void* data, unsigned long size, const void* deflt ) const;
  void Write( const TCHAR* value, const void* data, unsigned long size ) const;

private:
  HKEY key;

  // Return true if successful - used by the public read/write routines above
  bool ReadData( const TCHAR* value, void* data, unsigned long& size, unsigned long type ) const;
  bool WriteData( const TCHAR* value, const void* data, unsigned long size, unsigned long type ) const;

  // Disable copying
  RegistryKey( const RegistryKey& );
  RegistryKey& operator=( const RegistryKey& );
};

#endif // _h_registry_h_
