/******************************************************************************

  Winamp controller library

  Note: if this is used as a plugin, the Winamp window is actually available
  as part of the Winamp plugin configuration, so it is given to the class (via
  the SetWinamp() function); otherwise, the code searches for the window each
  time it is required. I chose to search because, if this is running as a
  separate application, Winamp could vanish and be restarted between calls, so
  caching the window handle would be risky.

                      Copyright 1999 Gavin Smyth
                 http://www.beesknees.freeserve.co.uk

******************************************************************************/

#include "common.h"
#include "winampmgr.h"
#include "winamp.h"

WinampManager::WinampManager() : winampWindow( NULL )
{
}

WinampManager::~WinampManager()
{
}

bool WinampManager::Exit()
{
  // I've found that occasionally shutting down Windows with Winamp still
  // active is bad news - possibly because of the VQF plugin - but whatever
  // the reason, it's better to exit Winamp, and then kill Windows
  SendWinamp( WM_CLOSE, 0, 0 );

  // It's of considerable assistance to debugging NOT to exit windows!
# ifndef _DEBUG
    Sleep( 1000 );                    // Give Winamp time to die
    ExitWindowsEx( EWX_POWEROFF | EWX_SHUTDOWN, 0 );
# endif

  return true;
}

// The action functions map fairly closely to the messages described in
// winamp.h, supplied by the Winamp folks

bool WinampManager::Volume( bool up )
{
  SendButton( up ? WINAMP_VOLUMEUP : WINAMP_VOLUMEDOWN );
  return true;
}

bool WinampManager::Skip( bool forward )
{
  SendButton( forward ? WINAMP_BUTTON5 : WINAMP_BUTTON1 );
  return true;
}

bool WinampManager::Step( bool forward )
{
  SendButton( forward ? WINAMP_FFWD5S : WINAMP_REW5S );
  return true;
}

bool WinampManager::SendButton( WPARAM button )
{
  return SendWinamp( WM_COMMAND, button, 0 );
}

bool WinampManager::SendWinamp( UINT message, WPARAM wparam, LPARAM lparam )
{
  HWND hwnd;
  if( winampWindow )
    hwnd = winampWindow;
  else
  {
    hwnd = FindWindow( _T("Winamp v1.x"), NULL );
    if( !hwnd )
      return false;
  }

  SendMessage( hwnd, message, wparam, lparam );
  return true;
}
